@@ -13,21 +13,35 @@
 # License along with this library; If not, see <http://www.gnu.org/licenses/>.
 #
 # Author: tasleson
+
 import hashlib
 
 import os
 import unittest
-import urlparse
 import re
 
 import sys
 import syslog
-import collections
-import inspect
-import urllib2
 
+try:
+    # Python 3.8 required change
+    from collections.abc import Sequence
+except ImportError:
+    from collections import Sequence
+
+import inspect
+
+try:
+    from urllib.error import (URLError, HTTPError)
+    from urllib.parse import urlparse
+except ImportError:
+    from urllib2 import (URLError, HTTPError)
+    from urlparse import urlparse
 import functools
 import traceback
+import six
+import ssl
+import socket
 
 
 def default_property(name, allow_set=True, doc=None):
@@ -56,27 +70,42 @@ def default_property(name, allow_set=True, doc=None):
 
 def common_urllib2_error_handler(exp):
 
-    if isinstance(exp, urllib2.HTTPError):
+    if isinstance(exp, HTTPError):
         raise LsmError(ErrorNumber.PLUGIN_AUTH_FAILED, str(exp))
-    if isinstance(exp, urllib2.URLError):
+    if isinstance(exp, URLError):
         desc = str(exp)
         if 'urlopen error' in desc:
             if 'Errno 111' in desc:
                 raise LsmError(ErrorNumber.NETWORK_CONNREFUSED,
                                'Connection refused')
             if 'Errno 113' in desc:
-                raise LsmError(ErrorNumber.NETWORK_HOSTDOWN,
-                               'Host is down')
+                raise LsmError(ErrorNumber.NETWORK_HOSTDOWN, 'Host is down')
         error("Unexpected network error:\n" + traceback.format_exc())
         raise LsmError(ErrorNumber.NETWORK_ERROR, desc)
 
+    try:
+        if ssl.CertificateError:
+            if isinstance(exp, ssl.CertificateError):
+                raise LsmError(ErrorNumber.NETWORK_ERROR,
+                               "SSL Certificate error (%s)" % str(exp))
+    except AttributeError:
+        pass
+
+    # Python3 is the gift that keeps on giving!
+    if six.PY3 and isinstance(exp, ConnectionError):
+        raise LsmError(ErrorNumber.NETWORK_CONNREFUSED, str(exp))
+
+    if isinstance(exp, socket.error):
+        raise LsmError(ErrorNumber.NETWORK_CONNREFUSED, str(exp))
+
     stack_trace = traceback.format_exc()
     error("Unexpected exception:\n" + stack_trace)
-    raise LsmError(ErrorNumber.PLUGIN_BUG, "Unexpected exception",
+    raise LsmError(ErrorNumber.PLUGIN_BUG,
+                   "Unexpected exception (TYPE= %s)" % str(type(exp)),
                    stack_trace)
 
 
-## Documentation for Proxy class.
+# Documentation for Proxy class.
 #
 # Class to encapsulate the actual class we want to call.  When an attempt is
 # made to access an attribute that doesn't exist we will raise an LsmError
@@ -86,7 +115,7 @@ class Proxy(object):
     Used to provide an unambiguous error when a feature is not implemented.
     """
 
-    ## The constructor.
+    # The constructor.
     # @param    self    The object self
     # @param    obj     The object instance to wrap
     def __init__(self, obj=None):
@@ -95,7 +124,7 @@ class Proxy(object):
         """
         self.proxied_obj = obj
 
-    ## Called each time an attribute is requested of the object
+    # Called each time an attribute is requested of the object
     # @param    self    The object self
     # @param    name    Name of the attribute being accessed
     # @return   The result of the method
@@ -106,10 +135,9 @@ class Proxy(object):
         if hasattr(self.proxied_obj, name):
             return functools.partial(self._present, name)
         else:
-            raise LsmError(ErrorNumber.NO_SUPPORT,
-                           "Unsupported operation")
+            raise LsmError(ErrorNumber.NO_SUPPORT, "Unsupported operation")
 
-    ## Method which is called to invoke the actual method of interest.
+    # Method which is called to invoke the actual method of interest.
     # @param    self                The object self
     # @param    _proxy_method_name  Method to invoke
     # @param    args                Arguments
@@ -121,44 +149,45 @@ class Proxy(object):
         """
         return getattr(self.proxied_obj, _proxy_method_name)(*args, **kwargs)
 
+
 # variable in client and specified on the command line for the daemon
 UDS_PATH = '/var/run/lsm/ipc'
 
-#Set to True for verbose logging
+# Set to True for verbose logging
 LOG_VERBOSE = True
 
-##Constant for byte size
+# Constant for byte size
 SIZE_CONS = {
     'B': 1,
-    'KiB': 2 ** 10,
-    'KB': 10 ** 3,
-    'K': 2 ** 10,
-    'k': 2 ** 10,
-    'MiB': 2 ** 20,
-    'MB': 10 ** 6,
-    'M': 2 ** 20,
-    'm': 2 ** 20,
-    'GiB': 2 ** 30,
-    'GB': 10 ** 9,
-    'G': 2 ** 30,
-    'g': 2 ** 30,
-    'TiB': 2 ** 40,
-    'TB': 10 ** 12,
-    'T': 2 ** 40,
-    't': 2 ** 40,
-    'PiB': 2 ** 50,
-    'PB': 10 ** 15,
-    'P': 2 ** 50,
-    'p': 2 ** 50,
-    'EiB': 2 ** 60,
-    'EB': 10 ** 18,
-    'E': 2 ** 60,
-    'e': 2 ** 60,
+    'KiB': 2**10,
+    'KB': 10**3,
+    'K': 2**10,
+    'k': 2**10,
+    'MiB': 2**20,
+    'MB': 10**6,
+    'M': 2**20,
+    'm': 2**20,
+    'GiB': 2**30,
+    'GB': 10**9,
+    'G': 2**30,
+    'g': 2**30,
+    'TiB': 2**40,
+    'TB': 10**12,
+    'T': 2**40,
+    't': 2**40,
+    'PiB': 2**50,
+    'PB': 10**15,
+    'P': 2**50,
+    'p': 2**50,
+    'EiB': 2**60,
+    'EB': 10**18,
+    'E': 2**60,
+    'e': 2**60,
 }
 SIZE_CONS_CHK_LST = ['EiB', 'PiB', 'TiB', 'GiB', 'MiB', 'KiB']
 
 
-##Converts the size into human format.
+# Converts the size into human format.
 # @param    size    Size in bytes
 # @param    human   True|False
 # @return Human representation of size
@@ -184,7 +213,7 @@ def sh(size, human=False):
         return size
 
 
-##Converts the size into human format.
+# Converts the size into human format.
 # @param  size    Size in bytes
 # @return Human representation of size in IEC binary size prefixes.
 def size_bytes_2_size_human(size):
@@ -198,7 +227,7 @@ def size_bytes_2_size_human(size):
     return sh(size, True)
 
 
-##Converts the size into human format.
+# Converts the size into human format.
 # @param size_human Human readable size string, e.g. '1.9 KiB'
 # @return Size in bytes
 def size_human_2_size_bytes(size_human):
@@ -214,7 +243,8 @@ def size_human_2_size_bytes(size_human):
         '2k'            # 2*(2**10), treated as '2KiB'
         '2KB'           # 2*(10**3)
     """
-    regex_size_human = re.compile(r"""
+    regex_size_human = re.compile(
+        r"""
         ^
         ([0-9\.]+)          # 1: number
         [ \t]*              # might have space between number and unit
@@ -222,8 +252,6 @@ def size_human_2_size_bytes(size_human):
         $
     """, re.X)
     regex_match = regex_size_human.match(size_human)
-    units = ''
-    number = 0
     size_bytes = 0
     if regex_match:
         number = regex_match.group(1)
@@ -237,7 +265,7 @@ def size_human_2_size_bytes(size_human):
     return int(size_bytes)
 
 
-## Common method used to parse a URI.
+# Common method used to parse a URI.
 # @param    uri         The uri to parse
 # @param    requires    Optional list of keys that must be present in output
 # @param    required_params Optional list of required parameters that
@@ -249,58 +277,62 @@ def uri_parse(uri, requires=None, required_params=None):
     before returning successfully.
     """
 
-    rc = {}
-    u = urlparse.urlparse(uri)
+    try:
+        rc = {}
+        u = urlparse(uri)
 
-    if u.scheme:
-        rc['scheme'] = u.scheme
+        if u.scheme:
+            rc['scheme'] = u.scheme
 
-    if u.netloc:
-        rc['netloc'] = u.netloc
+        if u.netloc:
+            rc['netloc'] = u.netloc
 
-    if u.port:
-        rc['port'] = u.port
+        if u.port:
+            rc['port'] = u.port
 
-    if u.hostname:
-        rc['host'] = u.hostname
+        if u.hostname:
+            rc['host'] = u.hostname
 
-    if u.username:
-        rc['username'] = u.username
-    else:
-        rc['username'] = None
+        if u.username:
+            rc['username'] = u.username
+        else:
+            rc['username'] = None
 
-    rc['parameters'] = uri_parameters(u)
+        rc['parameters'] = uri_parameters(u)
 
-    if requires:
-        for r in requires:
-            if r not in rc:
-                raise LsmError(ErrorNumber.PLUGIN_BUG,
-                               'uri missing \"%s\" or is in invalid form' % r)
+        if requires:
+            for r in requires:
+                if r not in rc:
+                    raise LsmError(
+                        ErrorNumber.INVALID_ARGUMENT,
+                        'uri missing \"%s\" or is in invalid form' % r)
 
-    if required_params:
-        for r in required_params:
-            if r not in rc['parameters']:
-                raise LsmError(ErrorNumber.PLUGIN_BUG,
-                               'uri missing query parameter %s' % r)
-    return rc
+        if required_params:
+            for r in required_params:
+                if r not in rc['parameters']:
+                    raise LsmError(ErrorNumber.INVALID_ARGUMENT,
+                                   'uri missing query parameter %s' % r)
+        return rc
+    except ValueError as ve:
+        raise LsmError(ErrorNumber.INVALID_ARGUMENT,
+                       "uri invalid: reason: %s" % str(ve))
 
 
-## Parses the parameters (Query string) of the URI
+# Parses the parameters (Query string) of the URI
 # @param    uri     Full uri
 # @returns  hash of the query string parameters.
 def uri_parameters(uri):
     # workaround for python bug:
     # http://bugs.python.org/issue9374
     # for URL: smispy+ssl://admin@emc-smi:5989?namespace=root/emc
-    # Before the patch commited( RHEL 6 and Fedora 18- ):
+    # Before the patch committed( RHEL 6 and Fedora 18- ):
     #       '?namespace=root/emc' is saved in uri.path
     # After patched(RHEL 7 and Fedora 19+):
     #       'namespace=root/emc' is saved in uri.query
-    query = ''
     if uri.query:
         query = uri.query
     elif uri.path:
-        query = urlparse.urlparse('http:' + uri[2]).query
+        query = urlparse('http:' + uri[2]).query
     else:
         return {}
     if query:
@@ -309,28 +341,43 @@ def uri_parameters(uri):
         return {}
 
 
-## Generates the md5 hex digest of passed in parameter.
+# Generates the md5 hex digest of passed in parameter.
 # @param    t   Item to generate signature on.
 # @returns  md5 hex digest.
 def md5(t):
-    h = hashlib.md5()
-    h.update(t)
+    try:
+        # The use of md5 is not used for security, indicate
+        # this to hashlib so that we can run when fips is enabled
+        h = hashlib.md5(usedforsecurity=False)
+    except Exception:
+        h = hashlib.md5()
+    h.update(t.encode("utf-8"))
     return h.hexdigest()
 
 
-## Converts a list of arguments to string.
+def int_div(a, b):
+    # Trying to avoid using past.old_div as we don't have future on all
+    # platforms we are trying to support
+    if six.PY3:
+        return a // b
+    else:
+        return int(a) / int(b)
+
+
+# Converts a list of arguments to string.
 # @param    args    Args to join
 # @return string of arguments joined together.
 def params_to_string(*args):
     return ''.join([str(e) for e in args])
 
+
 # Unfortunately the process name remains as 'python' so we are using argv[0] in
 # the output to allow us to determine which python exe is indeed logging to
 # syslog.
 # TODO:  On newer versions of python this is no longer true, need to fix.
 
 
-## Posts a message to the syslogger.
+# Posts a message to the syslogger.
 # @param    level   Logging level
 # @param    prg     Program name
 # @param    msg     Message to log.
@@ -363,10 +410,11 @@ class SocketEOF(Exception):
     pass
 
 
-@default_property('code', 'Error code')
-@default_property('msg', 'Error message')
-@default_property('data', 'Optional error data')
+@default_property('code', doc='Error code')
+@default_property('msg', doc='Error message')
+@default_property('data', doc='Optional error data')
 class LsmError(Exception):
+
     def __init__(self, code, message, data=None, *args, **kwargs):
         """
         Class represents an error.
@@ -389,8 +437,13 @@ def addl_error_data(domain, level, exception, debug=None, debug_data=None):
     """
     Used for gathering additional information about an error.
     """
-    return {'domain': domain, 'level': level, 'exception': exception,
-            'debug': debug, 'debug_data': debug_data}
+    return {
+        'domain': domain,
+        'level': level,
+        'exception': exception,
+        'debug': debug,
+        'debug_data': debug_data
+    }
 
 
 def get_class(class_name):
@@ -410,9 +463,9 @@ def get_class(class_name):
     return m
 
 
-#Note: Some of these don't make sense for python, but they do for other
-#Languages so we will be keeping them consistent even though we won't be
-#using them.
+# Note: Some of these don't make sense for python, but they do for other
+# Languages so we will be keeping them consistent even though we won't be
+# using them.
 class ErrorNumber(object):
     OK = 0
     LIB_BUG = 1
@@ -420,6 +473,7 @@ class ErrorNumber(object):
     JOB_STARTED = 7
     TIMEOUT = 11
     DAEMON_NOT_RUNNING = 12
+    PERMISSION_DENIED = 13
 
     NAME_CONFLICT = 50
     EXISTS_INITIATOR = 52
@@ -428,15 +482,16 @@ class ErrorNumber(object):
 
     NO_STATE_CHANGE = 125
 
-    NETWORK_CONNREFUSED = 140   # Host on network, but connection refused
-    NETWORK_HOSTDOWN = 141      # Host unreachable on network
-    NETWORK_ERROR = 142         # Generic network error
+    NETWORK_CONNREFUSED = 140  # Host on network, but connection refused
+    NETWORK_HOSTDOWN = 141  # Host unreachable on network
+    NETWORK_ERROR = 142  # Generic network error
 
     NO_MEMORY = 152
     NO_SUPPORT = 153
 
     # Deletion related errors
-    IS_MASKED = 160             # Volume is masked to access group.
+    IS_MASKED = 160  # Volume is masked to access group.
+    HAS_CHILD_DEPENDENCY = 161  # Volume/File system has child dependency.
 
     NOT_FOUND_ACCESS_GROUP = 200
     NOT_FOUND_FS = 201
@@ -453,13 +508,15 @@ class ErrorNumber(object):
     NO_SUPPORT_ONLINE_CHANGE = 250
     NO_SUPPORT_OFFLINE_CHANGE = 251
 
-    PLUGIN_AUTH_FAILED = 300    # Client supplied credential are incorrect
-    PLUGIN_IPC_FAIL = 301       # Inter-process communication between client &
-                                # out of process plug-in encountered connection
-                                # errors.
+    PLUGIN_AUTH_FAILED = 300  # Client supplied credential are incorrect
+
+    # Inter-process communication between client & out of process plug-in
+    # encountered connection errors
+    PLUGIN_IPC_FAIL = 301
+
+    # Incorrect permission on UNIX domain socket used for IPC
+    PLUGIN_SOCKET_PERMISSION = 307
 
-    PLUGIN_SOCKET_PERMISSION = 307  # Incorrect permission on UNIX domain
-                                    # socket used for IPC
     PLUGIN_NOT_EXIST = 311
 
     NOT_ENOUGH_SPACE = 350
@@ -468,23 +525,23 @@ class ErrorNumber(object):
     TRANSPORT_SERIALIZATION = 401
     TRANSPORT_INVALID_ARG = 402
 
-    LAST_INIT_IN_ACCESS_GROUP = 502
     # refuse to remove the last initiator from access group
+    LAST_INIT_IN_ACCESS_GROUP = 502
 
     UNSUPPORTED_SEARCH_KEY = 510
 
-    EMPTY_ACCESS_GROUP = 511    # volume_mask() will fail if access group
-                                # has no member/initiator.
+    # volume_mask() will fail if access group has no member/initiator.
+    EMPTY_ACCESS_GROUP = 511
 
-    POOL_NOT_READY = 512        # Pool is not ready for create/resize/etc
+    POOL_NOT_READY = 512  # Pool is not ready for create/resize/etc
 
-    DISK_NOT_FREE = 513     # Disk is not in DISK.STATUS_FREE status.
+    DISK_NOT_FREE = 513  # Disk is not in DISK.STATUS_FREE status.
 
     _LOCALS = locals()
 
     @staticmethod
     def error_number_to_str(error_no):
-        for error_str in ErrorNumber._LOCALS.keys():
+        for error_str in list(ErrorNumber._LOCALS.keys()):
             if ErrorNumber._LOCALS[error_str] == error_no:
                 return "%s(%d)" % (error_str, error_no)
         return "UNKNOWN_ERROR_NUMBER(%d)" % error_no
@@ -497,8 +554,8 @@ class JobStatus(object):
 
 
 def type_compare(method_name, exp_type, act_val):
-    if isinstance(exp_type, collections.Sequence):
-        if not isinstance(act_val, collections.Sequence):
+    if isinstance(exp_type, Sequence):
+        if not isinstance(act_val, Sequence):
             raise TypeError("%s call is returning a %s, but is "
                             "expecting a sequence" %
                             (method_name, str(type(act_val))))
@@ -516,13 +573,14 @@ def type_compare(method_name, exp_type, act_val):
         # A number of times a method will return None or some valid type,
         # only check on the type if the value is not None
         if exp_type != type(act_val) and act_val is not None:
-            if (exp_type == unicode and type(act_val) == str):
+            if (isinstance(exp_type, six.string_types)
+                    and isinstance(act_val, six.string_types)):
                 return
             if not inspect.isclass(exp_type) or \
                     not issubclass(type(act_val), exp_type):
-                raise TypeError('%s call expected: %s got: %s ' %
-                                (method_name, str(exp_type),
-                                 str(type(act_val))))
+                raise TypeError(
+                    '%s call expected: %s got: %s ' %
+                    (method_name, str(exp_type), str(type(act_val))))
 
 
 def return_requires(*types):
@@ -534,7 +592,9 @@ def return_requires(*types):
     needs to be language agnostic, so making sure we have the correct types
     is quite important.
     """
+
     def outer(func):
+
         @functools.wraps(func)
         def inner(*args, **kwargs):
             r = func(*args, **kwargs)
@@ -544,9 +604,9 @@ def return_requires(*types):
             # in this case we require that all the args are present.
             if len(types) > 1:
                 if len(r) != len(types):
-                        raise TypeError("%s call expected %d "
-                                        "return values, actual = %d" %
-                                        (func.__name__, len(types), len(r)))
+                    raise TypeError("%s call expected %d "
+                                    "return values, actual = %d" %
+                                    (func.__name__, len(types), len(r)))
 
                 type_compare(func.__name__, types, r)
             elif len(types) == 1:
@@ -554,11 +614,14 @@ def return_requires(*types):
                 type_compare(func.__name__, types[0], r)
 
             return r
+
         return inner
+
     return outer
 
 
 class TestCommon(unittest.TestCase):
+
     def setUp(self):
         pass
 
@@ -572,8 +635,8 @@ class TestCommon(unittest.TestCase):
         try:
             raise LsmError(10, 'Message', 'Data')
         except LsmError as e:
-            self.assertTrue(e.code == 10 and e.msg == 'Message' and
-                            e.data == 'Data')
+            self.assertTrue(e.code == 10 and e.msg == 'Message'
+                            and e.data == 'Data')
 
         ed = addl_error_data('domain', 'level', 'exception', 'debug',
                              'debug_data')
