@@ -1,4 +1,5 @@
 # Copyright (C) 2014 Red Hat, Inc.
+# (C) Copyright 2017 Hewlett Packard Enterprise Development LP
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
@@ -14,6 +15,11 @@
 #
 # Author: Gris Ge <fge@redhat.com>
 import sys
+from datetime import datetime
+
+from lsm import (size_bytes_2_size_human, LsmError, ErrorNumber, Battery,
+                 System, Pool, Disk, Volume, AccessGroup, FileSystem,
+                 FsSnapshot, NfsExport, TargetPort, LocalDisk)
 
 try:
     from collections import OrderedDict
@@ -21,16 +27,10 @@ except ImportError:
     # python 2.6 or earlier, use backport
     from ordereddict import OrderedDict
 
-from datetime import datetime
-
-from lsm import (size_bytes_2_size_human, LsmError, ErrorNumber,
-                 System, Pool, Disk, Volume, AccessGroup,
-                 FileSystem, FsSnapshot, NfsExport, TargetPort)
-
 BIT_MAP_STRING_SPLITTER = ','
 
 
-## Users are reporting errors with broken pipe when piping output
+# Users are reporting errors with broken pipe when piping output
 # to another program.  This appears to be related to this issue:
 # http://bugs.python.org/issue11380
 # Unable to reproduce, but hopefully this will address it.
@@ -47,7 +47,7 @@ def out(msg):
 def _bit_map_to_str(bit_map, conv_dict):
     rc = []
     bit_map = int(bit_map)
-    for cur_enum in conv_dict.keys():
+    for cur_enum in list(conv_dict.keys()):
         if cur_enum & bit_map:
             rc.append(conv_dict[cur_enum])
     # If there are no bits set we really don't need a string
@@ -57,16 +57,17 @@ def _bit_map_to_str(bit_map, conv_dict):
 
 
 def _enum_type_to_str(int_type, conv_dict):
-    rc = ''
     int_type = int(int_type)
 
-    if int_type in conv_dict.keys():
+    if int_type in list(conv_dict.keys()):
         return conv_dict[int_type]
     return 'Unknown(%d)' % int_type
 
 
 def _str_to_enum(type_str, conv_dict):
-    keys = [k for k, v in conv_dict.items() if v.lower() == type_str.lower()]
+    keys = [
+        k for k, v in list(conv_dict.items()) if v.lower() == type_str.lower()
+    ]
     if len(keys) > 0:
         return keys[0]
     raise LsmError(ErrorNumber.INVALID_ARGUMENT,
@@ -87,6 +88,16 @@ def system_status_to_str(system_status):
     return _bit_map_to_str(system_status, _SYSTEM_STATUS_CONV)
 
 
+_SYSTEM_MODE_CONV = {
+    System.MODE_HARDWARE_RAID: "HW RAID",
+    System.MODE_HBA: "HBA",
+}
+
+
+def system_mode_to_str(system_mode):
+    return _SYSTEM_MODE_CONV.get(system_mode, "")
+
+
 _POOL_STATUS_CONV = {
     Pool.STATUS_UNKNOWN: 'Unknown',
     Pool.STATUS_OK: 'OK',
@@ -236,10 +247,93 @@ def tgt_port_type_to_str(port_type):
     return _enum_type_to_str(port_type, _TGT_PORT_TYPE_CONV)
 
 
+def disk_rpm_to_str(rpm):
+    if rpm == '':
+        return "No Support"
+    if rpm == Disk.RPM_NO_SUPPORT:
+        return "No Support"
+    if rpm == Disk.RPM_UNKNOWN:
+        return "Unknown"
+    if rpm == Disk.RPM_NON_ROTATING_MEDIUM:
+        return "Non-Rotating Medium"
+    if rpm == Disk.RPM_ROTATING_UNKNOWN_SPEED:
+        return "Rotating Medium Unknown Speed"
+    return str(rpm)
+
+
+def disk_link_type_to_str(link_type):
+    if link_type == '':
+        return "No Support"
+    return _enum_type_to_str(link_type, LocalDiskInfo._LINK_TYPE_MAP)
+
+
+def disk_health_status_to_str(health_status):
+    if health_status == '':
+        return "No Support"
+    if health_status == Disk.HEALTH_STATUS_UNKNOWN:
+        return "Unknown"
+    if health_status == Disk.HEALTH_STATUS_FAIL:
+        return "Failure"
+    if health_status == Disk.HEALTH_STATUS_WARN:
+        return "Warning"
+    if health_status == Disk.HEALTH_STATUS_GOOD:
+        return "Good"
+
+
+_BATTERY_TYPE_CONV = {
+    Battery.TYPE_UNKNOWN: "Unknown",
+    Battery.TYPE_OTHER: "Other",
+    Battery.TYPE_CHEMICAL: "Chemical",
+    Battery.TYPE_CAPACITOR: "Capacitor",
+}
+
+
+def battery_type_to_str(battery_type):
+    return _enum_type_to_str(battery_type, _BATTERY_TYPE_CONV)
+
+
+_BATTERY_STATUS_CONV = {
+    Battery.STATUS_UNKNOWN: "Unknown",
+    Battery.STATUS_OTHER: "Other",
+    Battery.STATUS_OK: "OK",
+    Battery.STATUS_DISCHARGING: "Discharging",
+    Battery.STATUS_CHARGING: "Charging",
+    Battery.STATUS_LEARNING: "Learning",
+    Battery.STATUS_DEGRADED: "Degraded",
+    Battery.STATUS_ERROR: "Error",
+}
+
+
+def battery_status_to_str(battery_status):
+    return _bit_map_to_str(battery_status, _BATTERY_STATUS_CONV)
+
+
+_DISK_LED_STATUS_CONV = {
+    Disk.LED_STATUS_UNKNOWN: 'Unknown',
+    Disk.LED_STATUS_IDENT_ON: 'IDENT_ON',
+    Disk.LED_STATUS_IDENT_OFF: 'IDENT_OFF',
+    Disk.LED_STATUS_IDENT_UNKNOWN: 'IDENT_UNKNOWN',
+    Disk.LED_STATUS_FAULT_ON: 'FAULT_ON',
+    Disk.LED_STATUS_FAULT_OFF: 'FAULT_OFF',
+    Disk.LED_STATUS_FAULT_UNKNOWN: 'FAULT_UNKNOWN',
+}
+
+
+def disk_led_status_to_str(led_status):
+    return _bit_map_to_str(led_status, _DISK_LED_STATUS_CONV)
+
+
+def disk_link_speed_to_str(link_speed):
+    if link_speed == Disk.LINK_SPEED_UNKNOWN:
+        return "Unknown"
+    return "%.1f Gbps" % float(link_speed / 1000.0)
+
+
 class PlugData(object):
+
     def __init__(self, description, plugin_version):
-            self.desc = description
-            self.version = plugin_version
+        self.desc = description
+        self.version = plugin_version
 
 
 class VolumeRAIDInfo(object):
@@ -264,10 +358,11 @@ class VolumeRAIDInfo(object):
     }
 
     VOL_CREATE_RAID_TYPES_STR = [
-        'RAID0', 'RAID1', 'RAID5', 'RAID6', 'RAID10', 'RAID50', 'RAID60']
+        'RAID0', 'RAID1', 'RAID5', 'RAID6', 'RAID10', 'RAID50', 'RAID60'
+    ]
 
-    def __init__(self, vol_id, raid_type, strip_size, disk_count,
-                 min_io_size, opt_io_size):
+    def __init__(self, vol_id, raid_type, strip_size, disk_count, min_io_size,
+                 opt_io_size):
         self.vol_id = vol_id
         self.raid_type = raid_type
         self.strip_size = strip_size
@@ -300,17 +395,115 @@ class PoolRAIDInfo(object):
 
     @staticmethod
     def member_type_to_str(member_type):
-        return _enum_type_to_str(
-            member_type, PoolRAIDInfo._MEMBER_TYPE_MAP)
+        return _enum_type_to_str(member_type, PoolRAIDInfo._MEMBER_TYPE_MAP)
 
 
 class VcrCap(object):
+
     def __init__(self, system_id, raid_types, strip_sizes):
         self.system_id = system_id
         self.raid_types = raid_types
         self.strip_sizes = strip_sizes
 
 
+class LocalDiskInfo(object):
+    _LINK_TYPE_MAP = {
+        Disk.LINK_TYPE_NO_SUPPORT: "No Support",
+        Disk.LINK_TYPE_UNKNOWN: "Unknown",
+        Disk.LINK_TYPE_FC: "FC",
+        Disk.LINK_TYPE_SSA: "SSA",
+        Disk.LINK_TYPE_SBP: "SBP",
+        Disk.LINK_TYPE_SRP: "SRP",
+        Disk.LINK_TYPE_ISCSI: "iSCSI",
+        Disk.LINK_TYPE_SAS: "SAS",
+        Disk.LINK_TYPE_ADT: "ADT",
+        Disk.LINK_TYPE_ATA: "PATA/SATA",
+        Disk.LINK_TYPE_USB: "USB",
+        Disk.LINK_TYPE_SOP: "SCSI over PCIE",
+        Disk.LINK_TYPE_PCIE: "PCI-E",
+    }
+
+    def __init__(self, sd_path, vpd83, rpm, link_type, serial_num, led_status,
+                 link_speed, health_status):
+        self.sd_path = sd_path
+        self.vpd83 = vpd83
+        self.rpm = rpm
+        self.link_type = link_type
+        self.serial_num = serial_num
+        self.led_status = led_status
+        self.link_speed = link_speed
+        self.health_status = health_status
+
+
+class VolumeRAMCacheInfo(object):
+
+    _PHY_DISK_CACHE_STATUS_MAP = {
+        Volume.PHYSICAL_DISK_CACHE_USE_DISK_SETTING: "Use Disk Setting",
+        Volume.PHYSICAL_DISK_CACHE_ENABLED: "Enabled",
+        Volume.PHYSICAL_DISK_CACHE_DISABLED: "Disabled",
+        Volume.PHYSICAL_DISK_CACHE_UNKNOWN: "Unknown",
+    }
+
+    _W_CACHE_POLICY_MAP = {
+        Volume.WRITE_CACHE_POLICY_UNKNOWN: "Unknown",
+        Volume.WRITE_CACHE_POLICY_WRITE_BACK: "Write Back",
+        Volume.WRITE_CACHE_POLICY_AUTO: "Auto",
+        Volume.WRITE_CACHE_POLICY_WRITE_THROUGH: "Write Through",
+    }
+
+    _W_CACHE_STATUS_MAP = {
+        Volume.WRITE_CACHE_STATUS_UNKNOWN: "Unknown",
+        Volume.WRITE_CACHE_STATUS_WRITE_BACK: "Write Back",
+        Volume.WRITE_CACHE_STATUS_WRITE_THROUGH: "Write Through",
+    }
+
+    _R_CACHE_POLICY_MAP = {
+        Volume.READ_CACHE_POLICY_UNKNOWN: "Unknown",
+        Volume.READ_CACHE_POLICY_ENABLED: "Enabled",
+        Volume.READ_CACHE_POLICY_DISABLED: "Disabled",
+    }
+
+    _R_CACHE_STATUS_MAP = {
+        Volume.READ_CACHE_STATUS_UNKNOWN: "Unknown",
+        Volume.READ_CACHE_STATUS_ENABLED: "Enabled",
+        Volume.READ_CACHE_STATUS_DISABLED: "Disabled",
+    }
+
+    def __init__(self, vol_id, write_cache_policy, write_cache_status,
+                 read_cache_policy, read_cache_status, phy_disk_cache):
+        self.vol_id = vol_id
+        self.write_cache_policy = write_cache_policy
+        self.write_cache_status = write_cache_status
+        self.read_cache_policy = read_cache_policy
+        self.read_cache_status = read_cache_status
+        self.phy_disk_cache = phy_disk_cache
+
+    @staticmethod
+    def phy_disk_cache_status_to_str(phy_disk_cache):
+        return _enum_type_to_str(phy_disk_cache,
+                                 VolumeRAMCacheInfo._PHY_DISK_CACHE_STATUS_MAP)
+
+    @staticmethod
+    def w_cache_policy_to_str(w_cache_p):
+        return _enum_type_to_str(w_cache_p,
+                                 VolumeRAMCacheInfo._W_CACHE_POLICY_MAP)
+
+    @staticmethod
+    def w_cache_status_to_str(w_cache_status):
+        return _enum_type_to_str(w_cache_status,
+                                 VolumeRAMCacheInfo._W_CACHE_STATUS_MAP)
+
+    @staticmethod
+    def r_cache_policy_to_str(r_cache_p):
+        return _enum_type_to_str(r_cache_p,
+                                 VolumeRAMCacheInfo._R_CACHE_POLICY_MAP)
+
+    @staticmethod
+    def r_cache_status_to_str(r_cache_status):
+        return _enum_type_to_str(r_cache_status,
+                                 VolumeRAMCacheInfo._R_CACHE_STATUS_MAP)
+
+
 class DisplayData(object):
 
     def __init__(self):
@@ -331,6 +524,9 @@ class DisplayData(object):
     SYSTEM_HEADER['name'] = 'Name'
     SYSTEM_HEADER['status'] = 'Status'
     SYSTEM_HEADER['status_info'] = 'Info'
+    SYSTEM_HEADER['fw_version'] = "FW Ver"
+    SYSTEM_HEADER['mode'] = "Mode"
+    SYSTEM_HEADER['read_cache_pct'] = "Read Cache Percentage"
 
     SYSTEM_COLUMN_SKIP_KEYS = []
     # XXX_COLUMN_SKIP_KEYS contain a list of property should be skipped when
@@ -338,6 +534,7 @@ class DisplayData(object):
 
     SYSTEM_VALUE_CONV_ENUM = {
         'status': system_status_to_str,
+        'mode': system_mode_to_str,
     }
 
     SYSTEM_VALUE_CONV_HUMAN = []
@@ -405,12 +602,11 @@ class DisplayData(object):
     VOL_HEADER['admin_state'] = 'Disabled'
     VOL_HEADER['pool_id'] = 'Pool ID'
     VOL_HEADER['system_id'] = 'System ID'
+    VOL_HEADER['sd_paths'] = 'Disk Paths'  # This is appended by cmdline.py
 
     VOL_COLUMN_SKIP_KEYS = ['block_size', 'num_of_blocks']
 
-    VOL_VALUE_CONV_ENUM = {
-        'admin_state': vol_admin_state_to_str
-    }
+    VOL_VALUE_CONV_ENUM = {'admin_state': vol_admin_state_to_str}
 
     VOL_VALUE_CONV_HUMAN = ['size_bytes', 'block_size']
 
@@ -431,12 +627,19 @@ class DisplayData(object):
     DISK_HEADER['size_bytes'] = 'Size'
     DISK_HEADER['status'] = 'Status'
     DISK_HEADER['system_id'] = 'System ID'
+    DISK_HEADER['vpd83'] = 'SCSI VPD 0x83'
+    DISK_HEADER['sd_paths'] = 'Disk Paths'  # This is appended by cmdline.py
+    DISK_HEADER['rpm'] = 'Revolutions Per Minute'
+    DISK_HEADER['link_type'] = 'Link Type'
+    DISK_HEADER['location'] = 'Location'
 
-    DISK_COLUMN_SKIP_KEYS = ['block_size', 'num_of_blocks']
+    DISK_COLUMN_SKIP_KEYS = ['block_size', 'num_of_blocks', 'location']
 
     DISK_VALUE_CONV_ENUM = {
         'status': disk_status_to_str,
         'disk_type': disk_type_to_str,
+        'rpm': disk_rpm_to_str,
+        'link_type': disk_link_type_to_str,
     }
 
     DISK_VALUE_CONV_HUMAN = ['size_bytes', 'block_size']
@@ -482,8 +685,7 @@ class DisplayData(object):
 
     FS_COLUMN_SKIP_KEYS = []
 
-    FS_VALUE_CONV_ENUM = {
-    }
+    FS_VALUE_CONV_ENUM = {}
 
     FS_VALUE_CONV_HUMAN = ['total_space', 'free_space']
 
@@ -502,9 +704,7 @@ class DisplayData(object):
 
     FS_SNAP_COLUMN_SKIP_KEYS = []
 
-    FS_SNAP_VALUE_CONV_ENUM = {
-        'ts': datetime.fromtimestamp
-    }
+    FS_SNAP_VALUE_CONV_ENUM = {'ts': datetime.fromtimestamp}
 
     FS_SNAP_VALUE_CONV_HUMAN = []
 
@@ -578,9 +778,10 @@ class DisplayData(object):
 
     VOL_RAID_INFO_VALUE_CONV_ENUM = {
         'raid_type': VolumeRAIDInfo.raid_type_to_str,
-        }
+    }
     VOL_RAID_INFO_VALUE_CONV_HUMAN = [
-        'strip_size', 'min_io_size', 'opt_io_size']
+        'strip_size', 'min_io_size', 'opt_io_size'
+    ]
 
     VALUE_CONVERT[VolumeRAIDInfo] = {
         'headers': VOL_RAID_INFO_HEADER,
@@ -600,7 +801,7 @@ class DisplayData(object):
     POOL_RAID_INFO_VALUE_CONV_ENUM = {
         'raid_type': VolumeRAIDInfo.raid_type_to_str,
         'member_type': PoolRAIDInfo.member_type_to_str,
-        }
+    }
     POOL_RAID_INFO_VALUE_CONV_HUMAN = []
 
     VALUE_CONVERT[PoolRAIDInfo] = {
@@ -618,7 +819,8 @@ class DisplayData(object):
     VCR_CAP_COLUMN_SKIP_KEYS = []
 
     VCR_CAP_VALUE_CONV_ENUM = {
-        'raid_types': lambda i: [VolumeRAIDInfo.raid_type_to_str(x) for x in i]
+        'raid_types':
+        lambda i: [VolumeRAIDInfo.raid_type_to_str(x) for x in i]
     }
     VCR_CAP_VALUE_CONV_HUMAN = ['strip_sizes']
 
@@ -629,12 +831,96 @@ class DisplayData(object):
         'value_conv_human': VCR_CAP_VALUE_CONV_HUMAN,
     }
 
+    LOCAL_DISK_HEADER = OrderedDict()
+    LOCAL_DISK_HEADER['sd_path'] = 'Path'
+    LOCAL_DISK_HEADER['vpd83'] = 'SCSI VPD 0x83'
+    LOCAL_DISK_HEADER['rpm'] = 'Revolutions Per Minute'
+    LOCAL_DISK_HEADER['link_type'] = 'Link Type'
+    LOCAL_DISK_HEADER['serial_num'] = 'Serial Number'
+    LOCAL_DISK_HEADER['led_status'] = 'LED Status'
+    LOCAL_DISK_HEADER['link_speed'] = 'Link Speed'
+    LOCAL_DISK_HEADER['health_status'] = 'Health Status'
+
+    LOCAL_DISK_COLUMN_SKIP_KEYS = ['rpm', 'led_status', 'link_speed']
+
+    LOCAL_DISK_VALUE_CONV_ENUM = {
+        'rpm': disk_rpm_to_str,
+        'link_type': disk_link_type_to_str,
+        'led_status': disk_led_status_to_str,
+        'link_speed': disk_link_speed_to_str,
+        'health_status': disk_health_status_to_str,
+    }
+
+    LOCAL_DISK_VALUE_CONV_HUMAN = []
+
+    VALUE_CONVERT[LocalDiskInfo] = {
+        'headers': LOCAL_DISK_HEADER,
+        'column_skip_keys': LOCAL_DISK_COLUMN_SKIP_KEYS,
+        'value_conv_enum': LOCAL_DISK_VALUE_CONV_ENUM,
+        'value_conv_human': LOCAL_DISK_VALUE_CONV_HUMAN,
+    }
+
+    BATTERY_HEADER = OrderedDict()
+    BATTERY_HEADER['id'] = 'ID'
+    BATTERY_HEADER['name'] = 'Name'
+    BATTERY_HEADER['type'] = 'Type'
+    BATTERY_HEADER['status'] = 'Status'
+    BATTERY_HEADER['system_id'] = 'System ID'
+
+    BATTERY_COLUMN_SKIP_KEYS = []
+
+    BATTERY_VALUE_CONV_ENUM = {
+        'type': battery_type_to_str,
+        'status': battery_status_to_str,
+    }
+    BATTERY_VALUE_CONV_HUMAN = ['']
+
+    VALUE_CONVERT[Battery] = {
+        'headers': BATTERY_HEADER,
+        'column_skip_keys': BATTERY_COLUMN_SKIP_KEYS,
+        'value_conv_enum': BATTERY_VALUE_CONV_ENUM,
+        'value_conv_human': BATTERY_VALUE_CONV_HUMAN,
+    }
+
+    VOL_CACHE_INFO_HEADER = OrderedDict()
+    VOL_CACHE_INFO_HEADER['vol_id'] = 'Volume ID'
+    VOL_CACHE_INFO_HEADER['write_cache_policy'] = 'Write Cache Policy'
+    VOL_CACHE_INFO_HEADER['write_cache_status'] = 'Write Cache'
+    VOL_CACHE_INFO_HEADER['read_cache_policy'] = 'Read Cache Policy'
+    VOL_CACHE_INFO_HEADER['read_cache_status'] = 'Read Cache'
+    VOL_CACHE_INFO_HEADER['phy_disk_cache'] = 'Physical Disk Cache'
+
+    VOL_CACHE_INFO_COLUMN_SKIP_KEYS = []
+
+    VOL_CACHE_INFO_VALUE_CONV_ENUM = {
+        'write_cache_policy': VolumeRAMCacheInfo.w_cache_policy_to_str,
+        'write_cache_status': VolumeRAMCacheInfo.w_cache_status_to_str,
+        'read_cache_policy': VolumeRAMCacheInfo.r_cache_policy_to_str,
+        'read_cache_status': VolumeRAMCacheInfo.r_cache_status_to_str,
+        'phy_disk_cache': VolumeRAMCacheInfo.phy_disk_cache_status_to_str,
+    }
+    VOL_CACHE_INFO_VALUE_CONV_HUMAN = []
+
+    VALUE_CONVERT[VolumeRAMCacheInfo] = {
+        'headers': VOL_CACHE_INFO_HEADER,
+        'column_skip_keys': VOL_CACHE_INFO_COLUMN_SKIP_KEYS,
+        'value_conv_enum': VOL_CACHE_INFO_VALUE_CONV_ENUM,
+        'value_conv_human': VOL_CACHE_INFO_VALUE_CONV_HUMAN,
+    }
+
     @staticmethod
     def _get_man_pro_value(obj, key, value_conv_enum, value_conv_human,
                            flag_human, flag_enum):
-        value = getattr(obj, key)
+        try:
+            value = getattr(obj, key)
+        except LsmError as lsm_err:
+            if lsm_err.code == ErrorNumber.NO_SUPPORT:
+                value = ''
+            else:
+                raise lsm_err
+
         if not flag_enum:
-            if key in value_conv_enum.keys():
+            if key in list(value_conv_enum.keys()):
                 value = value_conv_enum[key](value)
         if flag_human:
             if key in value_conv_human:
@@ -654,8 +940,12 @@ class DisplayData(object):
         return max_width
 
     @staticmethod
-    def _data_dict_gen(obj, flag_human, flag_enum, display_way,
-                       extra_properties=None, flag_dsp_all_data=False):
+    def _data_dict_gen(obj,
+                       flag_human,
+                       flag_enum,
+                       display_way,
+                       extra_properties=None,
+                       flag_dsp_all_data=False):
         data_dict = OrderedDict()
         value_convert = DisplayData.VALUE_CONVERT[type(obj)]
         headers = value_convert['headers']
@@ -668,11 +958,11 @@ class DisplayData(object):
         display_keys = []
 
         if display_way == DisplayData.DISPLAY_WAY_COLUMN:
-            for key_name in headers.keys():
+            for key_name in list(headers.keys()):
                 if key_name not in value_convert['column_skip_keys']:
                     display_keys.append(key_name)
         elif display_way == DisplayData.DISPLAY_WAY_SCRIPT:
-            display_keys = headers.keys()
+            display_keys = list(headers.keys())
 
         if extra_properties:
             for extra_key_name in extra_properties:
@@ -681,16 +971,18 @@ class DisplayData(object):
 
         for key in display_keys:
             key_str = headers[key]
-            value = DisplayData._get_man_pro_value(
-                obj, key, value_conv_enum, value_conv_human, flag_human,
-                flag_enum)
+            value = DisplayData._get_man_pro_value(obj, key, value_conv_enum,
+                                                   value_conv_human,
+                                                   flag_human, flag_enum)
             data_dict[key_str] = value
 
         return data_dict
 
     @staticmethod
-    def display_data(objs, display_way=None,
-                     flag_human=True, flag_enum=False,
+    def display_data(objs,
+                     display_way=None,
+                     flag_human=True,
+                     flag_enum=False,
                      extra_properties=None,
                      splitter=None,
                      flag_with_header=True,
@@ -705,19 +997,20 @@ class DisplayData(object):
             splitter = DisplayData.DEFAULT_SPLITTER
 
         data_dict_list = []
-        if type(objs[0]) in DisplayData.VALUE_CONVERT.keys():
+        if type(objs[0]) in list(DisplayData.VALUE_CONVERT.keys()):
             for obj in objs:
-                data_dict = DisplayData._data_dict_gen(
-                    obj, flag_human, flag_enum, display_way,
-                    extra_properties, flag_dsp_all_data)
+                data_dict = DisplayData._data_dict_gen(obj, flag_human,
+                                                       flag_enum, display_way,
+                                                       extra_properties,
+                                                       flag_dsp_all_data)
                 data_dict_list.extend([data_dict])
         else:
             return None
         if display_way == DisplayData.DISPLAY_WAY_SCRIPT:
             DisplayData.display_data_script_way(data_dict_list, splitter)
         elif display_way == DisplayData.DISPLAY_WAY_COLUMN:
-            DisplayData._display_data_column_way(
-                data_dict_list, splitter, flag_with_header)
+            DisplayData._display_data_column_way(data_dict_list, splitter,
+                                                 flag_with_header)
         return True
 
     @staticmethod
@@ -726,14 +1019,13 @@ class DisplayData(object):
         value_column_width = 1
 
         for data_dict in data_dict_list:
-            for key_name in data_dict.keys():
+            for key_name in list(data_dict.keys()):
                 # find the max column width of key
                 cur_key_width = len(key_name)
                 if cur_key_width > key_column_width:
                     key_column_width = cur_key_width
                 # find the max column width of value
                 cur_value = data_dict[key_name]
-                cur_value_width = 0
                 if isinstance(cur_value, list):
                     if len(cur_value) == 0:
                         continue
@@ -743,14 +1035,11 @@ class DisplayData(object):
                 if cur_value_width > value_column_width:
                     value_column_width = cur_value_width
 
-        row_format = '%%-%ds%s%%-%ds' % (key_column_width,
-                                         splitter,
+        row_format = '%%-%ds%s%%-%ds' % (key_column_width, splitter,
                                          value_column_width)
-        sub_row_format = '%s%s%%-%ds' % (' ' * key_column_width,
-                                         splitter,
+        sub_row_format = '%s%s%%-%ds' % (' ' * key_column_width, splitter,
                                          value_column_width)
-        obj_splitter = '%s%s%s' % ('-' * key_column_width,
-                                   '-' * len(splitter),
+        obj_splitter = '%s%s%s' % ('-' * key_column_width, '-' * len(splitter),
                                    '-' * value_column_width)
 
         for data_dict in data_dict_list:
@@ -775,13 +1064,13 @@ class DisplayData(object):
             return
         two_d_list = []
 
-        item_count = len(data_dict_list[0].keys())
+        item_count = len(list(data_dict_list[0].keys()))
 
         # determine how many lines we will print
         row_width = 0
         for data_dict in data_dict_list:
             cur_max_wd = 0
-            for key_name in data_dict.keys():
+            for key_name in list(data_dict.keys()):
                 if isinstance(data_dict[key_name], list):
                     cur_row_width = len(data_dict[key_name])
                     if cur_row_width > cur_max_wd:
@@ -806,14 +1095,14 @@ class DisplayData(object):
         # header
         current_row_num = -1
         if flag_with_header:
-            two_d_list[0] = data_dict_list[0].keys()
+            two_d_list[0] = list(data_dict_list[0].keys())
             current_row_num = 0
 
         # Fill the 2D list with data_dict_list
         for data_dict in data_dict_list:
             current_row_num += 1
             save_row_num = current_row_num
-            values = data_dict.values()
+            values = list(data_dict.values())
             for index in range(0, len(values)):
                 value = values[index]
                 if isinstance(value, list):
